@@ -42,11 +42,15 @@ async fn main() -> std::io::Result<()> {
             parse_credentials(&credentials)
         } else if let Ok(credentials) = std::env::var("CREDENTIALS") {
             parse_credentials(&credentials)
+        } else if args.anonymous {
+            let credentials = "anonymous:".to_string();
+            parse_credentials(&credentials)
         } else {
             let credentials = rpassword::prompt_password(
                 "\nEnter administrator credentials or press [ENTER] to use OAuth: ",
             )
             .unwrap();
+
             if !credentials.is_empty() {
                 parse_credentials(&credentials)
             } else {
