@@ -933,6 +933,10 @@ impl QueueName {
     pub fn into_inner(self) -> [u8; 8] {
         self.0
     }
+
+    pub fn as_slice(&self) -> &[u8] {
+        &self.0
+    }
 }
 
 impl ArchivedQueueName {
@@ -941,6 +945,10 @@ impl ArchivedQueueName {
             .unwrap_or_default()
             .trim_end_matches('\0')
     }
+
+    pub fn as_slice(&self) -> &[u8] {
+        self.0.as_ref()
+    }
 }
 
 impl Default for QueueName {
