@@ -4,16 +4,12 @@
  * SPDX-License-Identifier: AGPL-3.0-only OR LicenseRef-SEL
  */
 
-use std::env;
-use std::io::{self, Write};
-
 use base64::Engine;
 use base64::engine::general_purpose;
+use std::env;
+use std::io::{self, Write};
 use store::write::{AnyClass, AnyKey, BatchBuilder, ValueClass};
-use store::{
-    Deserialize, IterateParams, SUBSPACE_BITMAP_ID, SUBSPACE_BITMAP_TAG, SUBSPACE_BITMAP_TEXT,
-    SUBSPACE_INDEXES, Store,
-};
+use store::{Deserialize, IterateParams, SUBSPACE_INDEXES, Store};
 
 const HELP: &str = concat!(
     "Stalwart Server v",
@@ -76,15 +72,7 @@ pub async fn store_console(store: Store) {
                                     key: to_key.collect::<Vec<_>>(),
                                 },
                             )
-                            .set_values(
-                                ![
-                                    SUBSPACE_INDEXES,
-                                    SUBSPACE_BITMAP_ID,
-                                    SUBSPACE_BITMAP_TAG,
-                                    SUBSPACE_BITMAP_TEXT,
-                                ]
-                                .contains(&from_subspace),
-                            ),
+                            .set_values(![SUBSPACE_INDEXES].contains(&from_subspace)),
                             |key, value| {
                                 print!("{}", char::from(from_subspace));
                                 print_escaped(key);
