@@ -23,6 +23,10 @@ use groupware::{
 use http_proto::HttpResponse;
 use hyper::StatusCode;
 use store::write::BatchBuilder;
+use store::{
+    ValueKey,
+    write::{AlignedBytes, Archive},
+};
 use trc::AddContext;
 use types::{
     acl::Acl,
@@ -437,7 +441,12 @@ async fn copy_card(
 ) -> crate::Result<HttpResponse> {
     // Fetch card
     let card_ = server
-        .get_archive(from_account_id, Collection::ContactCard, from_document_id)
+        .store()
+        .get_value::<Archive<AlignedBytes>>(ValueKey::archive(
+            from_account_id,
+            Collection::ContactCard,
+            from_document_id,
+        ))
         .await
         .caused_by(trc::location!())?
         .ok_or(DavError::Code(StatusCode::NOT_FOUND))?;
@@ -498,7 +507,12 @@ async fn copy_card(
     let response = if let Some(to_document_id) = to_document_id {
         // Overwrite card on destination
         let card_ = server
-            .get_archive(to_account_id, Collection::ContactCard, to_document_id)
+            .store()
+            .get_value::<Archive<AlignedBytes>>(ValueKey::archive(
+                to_account_id,
+                Collection::ContactCard,
+                to_document_id,
+            ))
             .await
             .caused_by(trc::location!())?;
         if let Some(card_) = card_ {
@@ -527,6 +541,7 @@ async fn copy_card(
         .commit_batch(batch)
         .await
         .caused_by(trc::location!())?;
+    server.notify_task_queue();
 
     response
 }
@@ -546,7 +561,12 @@ async fn move_card(
 ) -> crate::Result<HttpResponse> {
     // Fetch card
     let card_ = server
-        .get_archive(from_account_id, Collection::ContactCard, from_document_id)
+        .store()
+        .get_value::<Archive<AlignedBytes>>(ValueKey::archive(
+            from_account_id,
+            Collection::ContactCard,
+            from_document_id,
+        ))
         .await
         .caused_by(trc::location!())?
         .ok_or(DavError::Code(StatusCode::NOT_FOUND))?;
@@ -640,7 +660,12 @@ async fn move_card(
     let response = if let Some(to_document_id) = to_document_id {
         // Overwrite card on destination
         let card_ = server
-            .get_archive(to_account_id, Collection::ContactCard, to_document_id)
+            .store()
+            .get_value::<Archive<AlignedBytes>>(ValueKey::archive(
+                to_account_id,
+                Collection::ContactCard,
+                to_document_id,
+            ))
             .await
             .caused_by(trc::location!())?;
         if let Some(card_) = card_ {
@@ -669,6 +694,7 @@ async fn move_card(
         .commit_batch(batch)
         .await
         .caused_by(trc::location!())?;
+    server.notify_task_queue();
 
     response
 }
@@ -685,7 +711,12 @@ async fn rename_card(
 ) -> crate::Result<HttpResponse> {
     // Fetch card
     let card_ = server
-        .get_archive(account_id, Collection::ContactCard, document_id)
+        .store()
+        .get_value::<Archive<AlignedBytes>>(ValueKey::archive(
+            account_id,
+            Collection::ContactCard,
+            document_id,
+        ))
         .await
         .caused_by(trc::location!())?
         .ok_or(DavError::Code(StatusCode::NOT_FOUND))?;
@@ -713,6 +744,7 @@ async fn rename_card(
         .commit_batch(batch)
         .await
         .caused_by(trc::location!())?;
+    server.notify_task_queue();
 
     Ok(HttpResponse::new(StatusCode::CREATED))
 }
@@ -733,7 +765,12 @@ async fn copy_container(
 ) -> crate::Result<HttpResponse> {
     // Fetch book
     let book_ = server
-        .get_archive(from_account_id, Collection::AddressBook, from_document_id)
+        .store()
+        .get_value::<Archive<AlignedBytes>>(ValueKey::archive(
+            from_account_id,
+            Collection::AddressBook,
+            from_document_id,
+        ))
         .await
         .caused_by(trc::location!())?
         .ok_or(DavError::Code(StatusCode::NOT_FOUND))?;
@@ -774,7 +811,12 @@ async fn copy_container(
     let to_document_id = if let Some(to_document_id) = to_document_id {
         // Overwrite destination
         let book_ = server
-            .get_archive(to_account_id, Collection::AddressBook, to_document_id)
+            .store()
+            .get_value::<Archive<AlignedBytes>>(ValueKey::archive(
+                to_account_id,
+                Collection::AddressBook,
+                to_document_id,
+            ))
             .await
             .caused_by(trc::location!())?;
         if let Some(book_) = book_ {
@@ -811,11 +853,12 @@ async fn copy_container(
     let mut required_space = 0;
     for from_child_document_id in from_children_ids {
         if let Some(card_) = server
-            .get_archive(
+            .store()
+            .get_value::<Archive<AlignedBytes>>(ValueKey::archive(
                 from_account_id,
                 Collection::ContactCard,
                 from_child_document_id,
-            )
+            ))
             .await?
         {
             let card = card_
@@ -905,6 +948,7 @@ async fn copy_container(
         .commit_batch(batch)
         .await
         .caused_by(trc::location!())?;
+    server.notify_task_queue();
 
     if !is_overwrite {
         Ok(HttpResponse::new(StatusCode::CREATED))
@@ -924,7 +968,12 @@ async fn rename_container(
 ) -> crate::Result<HttpResponse> {
     // Fetch book
     let book_ = server
-        .get_archive(account_id, Collection::AddressBook, document_id)
+        .store()
+        .get_value::<Archive<AlignedBytes>>(ValueKey::archive(
+            account_id,
+            Collection::AddressBook,
+            document_id,
+        ))
         .await
         .caused_by(trc::location!())?
         .ok_or(DavError::Code(StatusCode::NOT_FOUND))?;
@@ -945,6 +994,7 @@ async fn rename_container(
         .commit_batch(batch)
         .await
         .caused_by(trc::location!())?;
+    server.notify_task_queue();
 
     Ok(HttpResponse::new(StatusCode::CREATED))
 }
