@@ -272,24 +272,19 @@ impl PrincipalField {
 }
 
 pub trait SpecialSecrets {
-    fn is_otp_auth(&self) -> bool;
-    fn is_app_password(&self) -> bool;
-    fn is_password(&self) -> bool;
+    fn is_otp_secret(&self) -> bool;
+    fn is_app_secret(&self) -> bool;
 }
 
 impl<T> SpecialSecrets for T
 where
     T: AsRef<str>,
 {
-    fn is_otp_auth(&self) -> bool {
+    fn is_otp_secret(&self) -> bool {
         self.as_ref().starts_with("otpauth://")
     }
 
-    fn is_app_password(&self) -> bool {
+    fn is_app_secret(&self) -> bool {
         self.as_ref().starts_with("$app$")
     }
-
-    fn is_password(&self) -> bool {
-        !self.is_otp_auth() && !self.is_app_password()
-    }
 }
