@@ -18,7 +18,7 @@ impl Permission {
             Permission::Impersonate => "Act on behalf of another user",
             Permission::UnlimitedRequests => "Perform unlimited requests",
             Permission::UnlimitedUploads => "Upload unlimited data",
-            Permission::DeleteSystemFolders => "Delete of system folders",
+            Permission::DeleteSystemFolders_ => "",
             Permission::MessageQueueList => "View message queue",
             Permission::MessageQueueGet => "Retrieve specific messages from the queue",
             Permission::MessageQueueUpdate => "Modify queued messages",
@@ -83,7 +83,7 @@ impl Permission {
             Permission::WebadminUpdate => "Modify web admin interface settings",
             Permission::LogsView => "Access system logs",
             Permission::SpamFilterTrain => "Train the spam filter",
-            Permission::SpamFilterClassify => "Classify emails with the spam filter",
+            Permission::SpamFilterTest => "Test the spam filter",
             Permission::Restart => "Restart the email server",
             Permission::TracingList => "View stored traces",
             Permission::TracingGet => "Retrieve specific trace information",
