@@ -4,31 +4,34 @@
  * SPDX-License-Identifier: AGPL-3.0-only OR LicenseRef-SEL
  */
 
-use std::net::IpAddr;
-
-use common::{Server, auth::AccessToken, config::spamfilter::SpamFilterAction, psl};
-
-use compact_str::CompactString;
+use common::{
+    Server,
+    auth::AccessToken,
+    config::spamfilter::SpamFilterAction,
+    manager::{SPAM_CLASSIFIER_KEY, SPAM_TRAINER_KEY},
+    psl,
+};
 use directory::{
     Permission,
     backend::internal::manage::{self, ManageDirectory},
 };
+use email::message::ingest::EmailIngest;
+use http_proto::{request::decode_path_element, *};
 use hyper::Method;
 use mail_auth::{
     AuthenticatedMessage, DmarcResult, dmarc::verify::DmarcParameters, spf::verify::SpfParameters,
 };
-use mail_parser::{Message, MessageParser};
+use mail_parser::MessageParser;
 use serde::{Deserialize, Serialize};
 use serde_json::json;
 use spam_filter::{
     SpamFilterInput,
     analysis::{init::SpamFilterInit, score::SpamFilterAnalyzeScore},
-    modules::bayes::BayesClassifier,
+    modules::classifier::SpamClassifier,
 };
 use std::future::Future;
-use store::ahash::AHashMap;
-
-use http_proto::{request::decode_path_element, *};
+use std::net::IpAddr;
+use store::{ahash::AHashMap, write::BatchBuilder};
 
 pub trait ManageSpamHandler: Sync + Send {
     fn handle_manage_spam(
@@ -66,8 +69,8 @@ pub struct SpamClassifyRequest {
 #[derive(Debug, Serialize, Deserialize)]
 #[serde(rename_all = "camelCase")]
 pub struct SpamClassifyResponse {
-    pub score: f64,
-    pub tags: AHashMap<CompactString, SpamFilterDisposition<f64>>,
+    pub score: f32,
+    pub tags: AHashMap<String, SpamFilterDisposition<f32>>,
     pub disposition: SpamFilterDisposition<String>,
 }
 
@@ -89,31 +92,107 @@ impl ManageSpamHandler for Server {
         session: &HttpSessionData,
         access_token: &AccessToken,
     ) -> trc::Result<HttpResponse> {
-        // Validate the access token
-        access_token.assert_has_permission(Permission::SpamFilterTrain)?;
-
         match (path.get(1).copied(), path.get(2).copied(), req.method()) {
-            (Some("train"), Some(class @ ("ham" | "spam")), &Method::POST) => {
-                let message = parse_message_or_err(body.as_deref().unwrap_or_default())?;
-                let input = if let Some(account) = path.get(3).copied().filter(|a| !a.is_empty()) {
-                    let account_id = self
+            (Some("upload"), Some(class @ ("ham" | "spam")), &Method::POST) => {
+                // Validate the access token
+                access_token.assert_has_permission(Permission::SpamFilterTrain)?;
+
+                let message =
+                    body.ok_or_else(|| manage::error("Failed to parse message.", None::<u64>))?;
+                let account_id = if let Some(account) =
+                    path.get(3).copied().filter(|a| !a.is_empty())
+                {
+                    let principal = self
                         .store()
-                        .get_principal_id(decode_path_element(account).as_ref())
+                        .get_principal_info(decode_path_element(account).as_ref())
                         .await?
                         .ok_or_else(|| manage::not_found(account.to_string()))?;
-                    SpamFilterInput::from_account_message(&message, account_id, session.session_id)
+                    if access_token.tenant.is_some() && principal.tenant != access_token.tenant_id()
+                    {
+                        return Err(manage::error(
+                            "Account does not belong to this tenant.",
+                            None::<u64>,
+                        ));
+                    }
+
+                    principal.id
+                } else if access_token.tenant.is_none() {
+                    u32::MAX
                 } else {
-                    SpamFilterInput::from_message(&message, session.session_id)
+                    return Err(manage::error(
+                        "Account ID is required for tenants.",
+                        None::<u64>,
+                    ));
                 };
-                self.bayes_train(&self.spam_filter_init(input), class == "spam", true)
-                    .await?;
+
+                // Write sample
+                let (blob_hash, blob_hold) =
+                    self.put_temporary_blob(account_id, &message, 60).await?;
+                let mut batch = BatchBuilder::new();
+                batch.with_account_id(account_id).clear(blob_hold);
+                self.add_spam_sample(
+                    &mut batch,
+                    blob_hash,
+                    class == "spam",
+                    true,
+                    session.session_id,
+                );
+                self.store().write(batch.build_all()).await?;
 
                 Ok(JsonResponse::new(json!({
                     "data": (),
                 }))
                 .into_http_response())
             }
+            (Some("train"), request, &Method::GET) => {
+                // Validate the access token
+                access_token.assert_has_permission(Permission::SpamFilterTrain)?;
+
+                let result = match request {
+                    Some("start") | Some("reset") => {
+                        if !self.inner.ipc.train_task_controller.is_running() {
+                            let reset = matches!(request, Some("reset"));
+                            let server = self.clone();
+                            tokio::spawn(async move {
+                                if let Err(err) = server.spam_train(reset).await {
+                                    trc::error!(err.caused_by(trc::location!()));
+                                }
+                            });
+
+                            true
+                        } else {
+                            false
+                        }
+                    }
+                    Some("stop") => {
+                        if self.inner.ipc.train_task_controller.is_running() {
+                            self.inner.ipc.train_task_controller.stop();
+                            true
+                        } else {
+                            false
+                        }
+                    }
+                    Some("delete") => {
+                        for key in [SPAM_CLASSIFIER_KEY, SPAM_TRAINER_KEY] {
+                            self.blob_store().delete_blob(key).await?;
+                        }
+                        true
+                    }
+                    Some("status") => self.inner.ipc.train_task_controller.is_running(),
+                    _ => {
+                        return Err(trc::ResourceEvent::NotFound.into_err());
+                    }
+                };
+
+                Ok(JsonResponse::new(json!({
+                    "data": result,
+                }))
+                .into_http_response())
+            }
             (Some("classify"), _, &Method::POST) => {
+                // Validate the access token
+                access_token.assert_has_permission(Permission::SpamFilterTest)?;
+
                 // Parse request
                 let request = serde_json::from_slice::<SpamClassifyRequest>(
                     body.as_deref().unwrap_or_default(),
@@ -123,7 +202,10 @@ impl ManageSpamHandler for Server {
                 })?;
 
                 // Built spam filter input
-                let message = parse_message_or_err(request.message.as_bytes())?;
+                let message = MessageParser::new()
+                    .parse(request.message.as_bytes())
+                    .filter(|m| m.root_part().headers().iter().any(|h| !h.name.is_other()))
+                    .ok_or_else(|| manage::error("Failed to parse message.", None::<u64>))?;
 
                 let remote_ip = request.remote_ip;
                 let ehlo_domain = request.ehlo_domain.to_lowercase();
@@ -242,8 +324,8 @@ impl ManageSpamHandler for Server {
                     env_from: &request.env_from,
                     env_from_flags: request.env_from_flags,
                     env_rcpt_to: request.env_rcpt_to.iter().map(String::as_str).collect(),
-                    account_id: None,
                     is_test: true,
+                    is_train: false,
                 };
 
                 // Classify
@@ -255,9 +337,14 @@ impl ManageSpamHandler for Server {
                     score: ctx.result.score,
                     tags: AHashMap::with_capacity(ctx.result.tags.len()),
                     disposition: match result {
-                        SpamFilterAction::Allow(value) => SpamFilterDisposition::Allow { value },
+                        SpamFilterAction::Allow(value) => SpamFilterDisposition::Allow {
+                            value: value.headers,
+                        },
                         SpamFilterAction::Discard => SpamFilterDisposition::Discard,
                         SpamFilterAction::Reject => SpamFilterDisposition::Reject,
+                        SpamFilterAction::Disabled => SpamFilterDisposition::Allow {
+                            value: String::new(),
+                        },
                     },
                 };
                 for tag in ctx.result.tags {
@@ -267,7 +354,9 @@ impl ManageSpamHandler for Server {
                         }
                         Some(SpamFilterAction::Discard) => SpamFilterDisposition::Discard,
                         Some(SpamFilterAction::Reject) => SpamFilterDisposition::Reject,
-                        None => SpamFilterDisposition::Allow { value: 0.0 },
+                        Some(SpamFilterAction::Disabled) | None => {
+                            SpamFilterDisposition::Allow { value: 0.0 }
+                        }
                     };
                     response.tags.insert(tag, disposition);
                 }
@@ -281,10 +370,3 @@ impl ManageSpamHandler for Server {
         }
     }
 }
-
-fn parse_message_or_err(bytes: &[u8]) -> trc::Result<Message<'_>> {
-    MessageParser::new()
-        .parse(bytes)
-        .filter(|m| m.root_part().headers().iter().any(|h| !h.name.is_other()))
-        .ok_or_else(|| manage::error("Failed to parse message.", None::<u64>))
-}
