@@ -29,7 +29,10 @@ use imap_proto::{
     receiver::Request,
 };
 use std::{sync::Arc, time::Instant};
-use store::write::{AlignedBytes, Archive, BatchBuilder};
+use store::{
+    ValueKey,
+    write::{AlignedBytes, Archive, BatchBuilder},
+};
 use trc::AddContext;
 use types::{
     acl::{Acl, AclGrant},
@@ -359,7 +362,7 @@ impl<T: SessionStream> Session<T> {
             batch
                 .with_account_id(mailbox_id.account_id)
                 .with_collection(Collection::Mailbox)
-                .update_document(mailbox_id.mailbox_id)
+                .with_document(mailbox_id.mailbox_id)
                 .custom(
                     ObjectIndexBuilder::new()
                         .with_changes(mailbox)
@@ -460,7 +463,12 @@ impl<T: SessionStream> SessionData<T> {
         if let Some(mailbox) = self.get_mailbox_by_name(&arguments.mailbox_name) {
             if let Some(values) = self
                 .server
-                .get_archive(mailbox.account_id, Collection::Mailbox, mailbox.mailbox_id)
+                .store()
+                .get_value::<Archive<AlignedBytes>>(ValueKey::archive(
+                    mailbox.account_id,
+                    Collection::Mailbox,
+                    mailbox.mailbox_id,
+                ))
                 .await
                 .caused_by(trc::location!())?
             {
