@@ -15,7 +15,6 @@ use email::{
     cache::{MessageCacheFetch, email::MessageCacheAccess},
     mailbox::{JUNK_ID, TRASH_ID, UidMailbox},
     message::{
-        bayes::EmailBayesTrain,
         copy::{CopyMessageError, EmailCopy},
         ingest::EmailIngest,
         metadata::MessageData,
@@ -27,9 +26,11 @@ use imap_proto::{
 };
 use std::{sync::Arc, time::Instant};
 use store::{
+    ValueKey,
     roaring::RoaringBitmap,
-    write::{AlignedBytes, Archive, BatchBuilder, ValueClass},
+    write::{AlignedBytes, Archive, BatchBuilder},
 };
+use trc::AddContext;
 use types::{
     acl::Acl,
     collection::{Collection, VanishedCollection},
@@ -204,8 +205,6 @@ impl<T: SessionStream> SessionData<T> {
             // Mailboxes are in the same account
             let account_id = src_mailbox.id.account_id;
             let dest_mailbox_id = UidMailbox::new_unassigned(dest_mailbox_id);
-            let can_spam_train = self.server.email_bayes_can_train(&access_token);
-            let mut has_spam_train_tasks = false;
             let mut batch = BatchBuilder::new();
 
             for (id, imap_id) in ids {
@@ -245,18 +244,16 @@ impl<T: SessionStream> SessionData<T> {
                     copied_ids.push((imap_id.uid, mailbox.uid.to_native()));
 
                     if is_move {
-                        let mut new_data = data
-                            .deserialize()
-                            .imap_ctx(&arguments.tag, trc::location!())?;
+                        let mut new_data = data.inner.to_builder();
                         new_data.remove_mailbox(src_mailbox.id.mailbox_id);
                         batch
                             .with_account_id(account_id)
                             .with_collection(Collection::Email)
-                            .update_document(id)
+                            .with_document(id)
                             .custom(
                                 ObjectIndexBuilder::new()
                                     .with_current(data)
-                                    .with_changes(new_data),
+                                    .with_changes(new_data.seal()),
                             )
                             .imap_ctx(&arguments.tag, trc::location!())?
                             .log_vanished_item(
@@ -271,9 +268,7 @@ impl<T: SessionStream> SessionData<T> {
                 }
 
                 // Prepare changes
-                let mut new_data = data
-                    .deserialize()
-                    .imap_ctx(&arguments.tag, trc::location!())?;
+                let mut new_data = data.inner.to_builder();
 
                 // Add destination folder
                 new_data.add_mailbox(dest_mailbox_id);
@@ -282,28 +277,39 @@ impl<T: SessionStream> SessionData<T> {
                 }
 
                 // Assign IMAP UIDs
-                for uid_mailbox in &mut new_data.mailboxes {
-                    if uid_mailbox.uid == 0 {
-                        let assigned_uid = self
-                            .server
-                            .assign_imap_uid(account_id, uid_mailbox.mailbox_id)
-                            .await
-                            .imap_ctx(&arguments.tag, trc::location!())?;
-                        debug_assert!(assigned_uid > 0);
-                        copied_ids.push((imap_id.uid, assigned_uid));
-                        uid_mailbox.uid = assigned_uid;
-                    }
+                let ids = self
+                    .server
+                    .assign_email_ids(
+                        account_id,
+                        new_data
+                            .mailboxes
+                            .iter()
+                            .filter(|m| m.uid == 0)
+                            .map(|m| m.mailbox_id),
+                        false,
+                    )
+                    .await
+                    .caused_by(trc::location!())?;
+
+                for (uid_mailbox, uid) in new_data
+                    .mailboxes
+                    .iter_mut()
+                    .filter(|m| m.uid == 0)
+                    .zip(ids)
+                {
+                    copied_ids.push((imap_id.uid, uid));
+                    uid_mailbox.uid = uid;
                 }
 
                 // Prepare write batch
                 batch
                     .with_account_id(account_id)
                     .with_collection(Collection::Email)
-                    .update_document(id)
+                    .with_document(id)
                     .custom(
                         ObjectIndexBuilder::new()
                             .with_current(data)
-                            .with_changes(new_data),
+                            .with_changes(new_data.seal()),
                     )
                     .imap_ctx(&arguments.tag, trc::location!())?;
                 if is_move {
@@ -313,34 +319,21 @@ impl<T: SessionStream> SessionData<T> {
                     );
                 }
 
-                // Add bayes train task
-                if can_spam_train {
-                    if dest_mailbox_id.mailbox_id == JUNK_ID {
-                        batch.set(
-                            ValueClass::TaskQueue(
-                                self.server
-                                    .email_bayes_queue_task_build(account_id, id, true)
-                                    .await
-                                    .imap_ctx(&arguments.tag, trc::location!())?,
-                            ),
-                            vec![],
-                        );
-                        has_spam_train_tasks = true;
-                    } else if src_mailbox.id.mailbox_id == JUNK_ID
-                        && dest_mailbox_id.mailbox_id != TRASH_ID
-                    {
-                        batch.set(
-                            ValueClass::TaskQueue(
-                                self.server
-                                    .email_bayes_queue_task_build(account_id, id, false)
-                                    .await
-                                    .imap_ctx(&arguments.tag, trc::location!())?,
-                            ),
-                            vec![],
-                        );
-                        has_spam_train_tasks = true;
-                    }
+                // Add message to training queue
+                if dest_mailbox_id.mailbox_id == JUNK_ID {
+                    self.server
+                        .add_account_spam_sample(&mut batch, account_id, id, true, self.session_id)
+                        .await
+                        .imap_ctx(&arguments.tag, trc::location!())?;
+                } else if src_mailbox.id.mailbox_id == JUNK_ID
+                    && dest_mailbox_id.mailbox_id != TRASH_ID
+                {
+                    self.server
+                        .add_account_spam_sample(&mut batch, account_id, id, false, self.session_id)
+                        .await
+                        .imap_ctx(&arguments.tag, trc::location!())?;
                 }
+
                 batch.commit_point();
 
                 // Update changelog
@@ -354,11 +347,6 @@ impl<T: SessionStream> SessionData<T> {
                 .commit_batch(batch)
                 .await
                 .imap_ctx(&arguments.tag, trc::location!())?;
-
-            // Trigger Bayes training
-            if has_spam_train_tasks {
-                self.server.notify_task_queue();
-            }
         } else {
             // Obtain quota for target account
             let src_account_id = src_mailbox.id.account_id;
@@ -562,7 +550,12 @@ impl<T: SessionStream> SessionData<T> {
     ) -> trc::Result<Option<Archive<AlignedBytes>>> {
         if let Some(data) = self
             .server
-            .get_archive(account_id, Collection::Email, id)
+            .store()
+            .get_value::<Archive<AlignedBytes>>(ValueKey::archive(
+                account_id,
+                Collection::Email,
+                id,
+            ))
             .await?
         {
             Ok(Some(data))
