@@ -14,7 +14,10 @@ use imap_proto::{
     Command, ResponseCode, StatusResponse, protocol::rename::Arguments, receiver::Request,
 };
 use std::time::Instant;
-use store::write::BatchBuilder;
+use store::{
+    ValueKey,
+    write::{AlignedBytes, Archive, BatchBuilder},
+};
 use trc::AddContext;
 use types::{acl::Acl, collection::Collection};
 
@@ -85,7 +88,12 @@ impl<T: SessionStream> SessionData<T> {
         // Obtain mailbox
         let mailbox_ = self
             .server
-            .get_archive(params.account_id, Collection::Mailbox, mailbox_id)
+            .store()
+            .get_value::<Archive<AlignedBytes>>(ValueKey::archive(
+                params.account_id,
+                Collection::Mailbox,
+                mailbox_id,
+            ))
             .await
             .imap_ctx(&arguments.tag, trc::location!())?
             .ok_or_else(|| {
@@ -144,7 +152,7 @@ impl<T: SessionStream> SessionData<T> {
             batch
                 .with_account_id(params.account_id)
                 .with_collection(Collection::Mailbox)
-                .create_document(mailbox_id)
+                .with_document(mailbox_id)
                 .custom(ObjectIndexBuilder::<(), _>::new().with_changes(
                     email::mailbox::Mailbox::new(path_item).with_parent_id(parent_id),
                 ))
@@ -164,7 +172,7 @@ impl<T: SessionStream> SessionData<T> {
         batch
             .with_account_id(params.account_id)
             .with_collection(Collection::Mailbox)
-            .update_document(mailbox_id)
+            .with_document(mailbox_id)
             .custom(
                 ObjectIndexBuilder::new()
                     .with_current(mailbox)
