@@ -4,21 +4,38 @@
  * SPDX-License-Identifier: AGPL-3.0-only OR LicenseRef-SEL
  */
 
-use std::{sync::Arc, time::Instant};
-
 use crate::{
+    addressbook::{get::AddressBookGet, set::AddressBookSet},
     api::auth::JmapAuthorization,
     blob::{copy::BlobCopy, get::BlobOperations, upload::BlobUpload},
+    calendar::{get::CalendarGet, set::CalendarSet},
+    calendar_event::{
+        copy::JmapCalendarEventCopy, get::CalendarEventGet, parse::CalendarEventParse,
+        query::CalendarEventQuery, set::CalendarEventSet,
+    },
+    calendar_event_notification::{
+        get::CalendarEventNotificationGet, query::CalendarEventNotificationQuery,
+        set::CalendarEventNotificationSet,
+    },
     changes::{get::ChangesLookup, query::QueryChanges},
+    contact::{
+        copy::JmapContactCardCopy, get::ContactCardGet, parse::ContactCardParse,
+        query::ContactCardQuery, set::ContactCardSet,
+    },
     email::{
         copy::JmapEmailCopy, get::EmailGet, import::EmailImport, parse::EmailParse,
         query::EmailQuery, set::EmailSet, snippet::EmailSearchSnippet,
     },
+    file::{get::FileNodeGet, query::FileNodeQuery, set::FileNodeSet},
     identity::{get::IdentityGet, set::IdentitySet},
     mailbox::{get::MailboxGet, query::MailboxQuery, set::MailboxSet},
-    principal::{get::PrincipalGet, query::PrincipalQuery},
+    participant_identity::{get::ParticipantIdentityGet, set::ParticipantIdentitySet},
+    principal::{availability::PrincipalGetAvailability, get::PrincipalGet, query::PrincipalQuery},
     push::{get::PushSubscriptionFetch, set::PushSubscriptionSet},
     quota::{get::QuotaGet, query::QuotaQuery},
+    share_notification::{
+        get::ShareNotificationGet, query::ShareNotificationQuery, set::ShareNotificationSet,
+    },
     sieve::{
         get::SieveScriptGet, query::SieveScriptQuery, set::SieveScriptSet,
         validate::SieveScriptValidate,
@@ -31,16 +48,16 @@ use common::{Server, auth::AccessToken};
 use http_proto::HttpSessionData;
 use jmap_proto::{
     request::{
-        Call, CopyRequestMethod, GetRequestMethod, QueryRequestMethod, Request, RequestMethod,
-        SetRequestMethod, method::MethodName,
+        Call, CopyRequestMethod, GetRequestMethod, ParseRequestMethod, QueryRequestMethod, Request,
+        RequestMethod, SetRequestMethod, method::MethodName,
     },
     response::{Response, ResponseMethod, SetResponseMethod},
 };
+use std::future::Future;
+use std::{sync::Arc, time::Instant};
 use trc::JmapEvent;
 use types::{collection::Collection, id::Id};
 
-use std::future::Future;
-
 pub trait RequestHandler: Sync + Send {
     fn handle_jmap_request<'x>(
         &self,
@@ -126,6 +143,28 @@ impl RequestHandler for Server {
                                     SetResponseMethod::VacationResponse(set_response) => {
                                         set_response.update_created_ids(&mut response);
                                     }
+                                    SetResponseMethod::AddressBook(set_response) => {
+                                        set_response.update_created_ids(&mut response);
+                                    }
+                                    SetResponseMethod::ContactCard(set_response) => {
+                                        set_response.update_created_ids(&mut response);
+                                    }
+                                    SetResponseMethod::FileNode(set_response) => {
+                                        set_response.update_created_ids(&mut response);
+                                    }
+                                    SetResponseMethod::ShareNotification(set_response) => {
+                                        set_response.update_created_ids(&mut response);
+                                    }
+                                    SetResponseMethod::Calendar(set_response) => {
+                                        set_response.update_created_ids(&mut response);
+                                    }
+                                    SetResponseMethod::CalendarEvent(set_response) => {
+                                        set_response.update_created_ids(&mut response);
+                                    }
+                                    SetResponseMethod::ParticipantIdentity(set_response) => {
+                                        set_response.update_created_ids(&mut response);
+                                    }
+                                    SetResponseMethod::CalendarEventNotification(_) => {}
                                 }
                             }
                             ResponseMethod::ImportEmail(import_response) => {
@@ -235,7 +274,9 @@ impl RequestHandler for Server {
 
                     self.vacation_response_get(req).await?.into()
                 }
-                GetRequestMethod::Principal(req) => self.principal_get(req).await?.into(),
+                GetRequestMethod::Principal(req) => {
+                    self.principal_get(req, access_token).await?.into()
+                }
                 GetRequestMethod::Quota(mut req) => {
                     set_account_id_if_missing(&mut req.account_id, access_token);
                     access_token.assert_is_member(req.account_id)?;
@@ -248,6 +289,60 @@ impl RequestHandler for Server {
 
                     self.blob_get(req, access_token).await?.into()
                 }
+                GetRequestMethod::AddressBook(mut req) => {
+                    set_account_id_if_missing(&mut req.account_id, access_token);
+                    access_token.assert_has_access(req.account_id, Collection::AddressBook)?;
+
+                    self.address_book_get(req, access_token).await?.into()
+                }
+                GetRequestMethod::ContactCard(mut req) => {
+                    set_account_id_if_missing(&mut req.account_id, access_token);
+                    access_token.assert_has_access(req.account_id, Collection::ContactCard)?;
+
+                    self.contact_card_get(req, access_token).await?.into()
+                }
+                GetRequestMethod::FileNode(mut req) => {
+                    set_account_id_if_missing(&mut req.account_id, access_token);
+                    access_token.assert_has_access(req.account_id, Collection::FileNode)?;
+
+                    self.file_node_get(req, access_token).await?.into()
+                }
+                GetRequestMethod::PrincipalAvailability(req) => self
+                    .principal_get_availability(req, access_token)
+                    .await?
+                    .into(),
+                GetRequestMethod::Calendar(mut req) => {
+                    set_account_id_if_missing(&mut req.account_id, access_token);
+                    access_token.assert_has_access(req.account_id, Collection::Calendar)?;
+
+                    self.calendar_get(req, access_token).await?.into()
+                }
+                GetRequestMethod::CalendarEvent(mut req) => {
+                    set_account_id_if_missing(&mut req.account_id, access_token);
+                    access_token.assert_has_access(req.account_id, Collection::CalendarEvent)?;
+
+                    self.calendar_event_get(req, access_token).await?.into()
+                }
+                GetRequestMethod::CalendarEventNotification(mut req) => {
+                    set_account_id_if_missing(&mut req.account_id, access_token);
+                    access_token.assert_is_member(req.account_id)?;
+
+                    self.calendar_event_notification_get(req, access_token)
+                        .await?
+                        .into()
+                }
+                GetRequestMethod::ParticipantIdentity(mut req) => {
+                    set_account_id_if_missing(&mut req.account_id, access_token);
+                    access_token.assert_is_member(req.account_id)?;
+
+                    self.participant_identity_get(req).await?.into()
+                }
+                GetRequestMethod::ShareNotification(mut req) => {
+                    set_account_id_if_missing(&mut req.account_id, access_token);
+                    access_token.assert_is_member(req.account_id)?;
+
+                    self.share_notification_get(req).await?.into()
+                }
             },
             RequestMethod::Query(req) => match req {
                 QueryRequestMethod::Email(mut req) => {
@@ -274,16 +369,48 @@ impl RequestHandler for Server {
 
                     self.sieve_script_query(req).await?.into()
                 }
-                QueryRequestMethod::Principal(mut req) => {
-                    set_account_id_if_missing(&mut req.account_id, access_token);
-                    self.principal_query(req, session).await?.into()
-                }
+                QueryRequestMethod::Principal(req) => self
+                    .principal_query(req, access_token, session)
+                    .await?
+                    .into(),
                 QueryRequestMethod::Quota(mut req) => {
                     set_account_id_if_missing(&mut req.account_id, access_token);
                     access_token.assert_is_member(req.account_id)?;
 
                     self.quota_query(req, access_token).await?.into()
                 }
+                QueryRequestMethod::ContactCard(mut req) => {
+                    set_account_id_if_missing(&mut req.account_id, access_token);
+                    access_token.assert_has_access(req.account_id, Collection::ContactCard)?;
+
+                    self.contact_card_query(req, access_token).await?.into()
+                }
+                QueryRequestMethod::FileNode(mut req) => {
+                    set_account_id_if_missing(&mut req.account_id, access_token);
+                    access_token.assert_has_access(req.account_id, Collection::FileNode)?;
+
+                    self.file_node_query(req, access_token).await?.into()
+                }
+                QueryRequestMethod::CalendarEvent(mut req) => {
+                    set_account_id_if_missing(&mut req.account_id, access_token);
+                    access_token.assert_has_access(req.account_id, Collection::CalendarEvent)?;
+
+                    self.calendar_event_query(req, access_token).await?.into()
+                }
+                QueryRequestMethod::CalendarEventNotification(mut req) => {
+                    set_account_id_if_missing(&mut req.account_id, access_token);
+                    access_token.assert_is_member(req.account_id)?;
+
+                    self.calendar_event_notification_query(req, access_token)
+                        .await?
+                        .into()
+                }
+                QueryRequestMethod::ShareNotification(mut req) => {
+                    set_account_id_if_missing(&mut req.account_id, access_token);
+                    access_token.assert_is_member(req.account_id)?;
+
+                    self.share_notification_query(req).await?.into()
+                }
             },
             RequestMethod::Set(req) => match req {
                 SetRequestMethod::Email(mut req) => {
@@ -302,7 +429,7 @@ impl RequestHandler for Server {
                     set_account_id_if_missing(&mut req.account_id, access_token);
                     access_token.assert_is_member(req.account_id)?;
 
-                    self.identity_set(req).await?.into()
+                    self.identity_set(req, access_token).await?.into()
                 }
                 SetRequestMethod::EmailSubmission(mut req) => {
                     set_account_id_if_missing(&mut req.account_id, access_token);
@@ -330,6 +457,64 @@ impl RequestHandler for Server {
 
                     self.vacation_response_set(req, access_token).await?.into()
                 }
+                SetRequestMethod::AddressBook(mut req) => {
+                    set_account_id_if_missing(&mut req.account_id, access_token);
+                    access_token.assert_has_access(req.account_id, Collection::AddressBook)?;
+
+                    self.address_book_set(req, access_token, session)
+                        .await?
+                        .into()
+                }
+                SetRequestMethod::ContactCard(mut req) => {
+                    set_account_id_if_missing(&mut req.account_id, access_token);
+                    access_token.assert_has_access(req.account_id, Collection::ContactCard)?;
+
+                    self.contact_card_set(req, access_token, session)
+                        .await?
+                        .into()
+                }
+                SetRequestMethod::FileNode(mut req) => {
+                    set_account_id_if_missing(&mut req.account_id, access_token);
+                    access_token.assert_has_access(req.account_id, Collection::FileNode)?;
+
+                    self.file_node_set(req, access_token, session).await?.into()
+                }
+                SetRequestMethod::ShareNotification(mut req) => {
+                    set_account_id_if_missing(&mut req.account_id, access_token);
+                    access_token.assert_is_member(req.account_id)?;
+
+                    self.share_notification_set(req).await?.into()
+                }
+                SetRequestMethod::Calendar(mut req) => {
+                    set_account_id_if_missing(&mut req.account_id, access_token);
+                    access_token.assert_has_access(req.account_id, Collection::Calendar)?;
+
+                    self.calendar_set(req, access_token, session).await?.into()
+                }
+                SetRequestMethod::CalendarEvent(mut req) => {
+                    set_account_id_if_missing(&mut req.account_id, access_token);
+                    access_token.assert_has_access(req.account_id, Collection::CalendarEvent)?;
+
+                    self.calendar_event_set(req, access_token, session)
+                        .await?
+                        .into()
+                }
+                SetRequestMethod::CalendarEventNotification(mut req) => {
+                    set_account_id_if_missing(&mut req.account_id, access_token);
+                    access_token.assert_is_member(req.account_id)?;
+
+                    self.calendar_event_notification_set(req, access_token, session)
+                        .await?
+                        .into()
+                }
+                SetRequestMethod::ParticipantIdentity(mut req) => {
+                    set_account_id_if_missing(&mut req.account_id, access_token);
+                    access_token.assert_is_member(req.account_id)?;
+
+                    self.participant_identity_set(req, access_token)
+                        .await?
+                        .into()
+                }
             },
             RequestMethod::Changes(mut req) => {
                 set_account_id_if_missing(&mut req.account_id, access_token);
@@ -341,6 +526,7 @@ impl RequestHandler for Server {
             RequestMethod::Copy(req) => match req {
                 CopyRequestMethod::Email(mut req) => {
                     set_account_id_if_missing(&mut req.account_id, access_token);
+                    set_account_id_if_missing(&mut req.from_account_id, access_token);
 
                     access_token
                         .assert_has_access(req.account_id, Collection::Email)?
@@ -356,6 +542,30 @@ impl RequestHandler for Server {
 
                     self.blob_copy(req, access_token).await?.into()
                 }
+                CopyRequestMethod::ContactCard(mut req) => {
+                    set_account_id_if_missing(&mut req.from_account_id, access_token);
+                    set_account_id_if_missing(&mut req.account_id, access_token);
+
+                    access_token
+                        .assert_has_access(req.account_id, Collection::ContactCard)?
+                        .assert_has_access(req.from_account_id, Collection::ContactCard)?;
+
+                    self.contact_card_copy(req, access_token, next_call, session)
+                        .await?
+                        .into()
+                }
+                CopyRequestMethod::CalendarEvent(mut req) => {
+                    set_account_id_if_missing(&mut req.from_account_id, access_token);
+                    set_account_id_if_missing(&mut req.account_id, access_token);
+
+                    access_token
+                        .assert_has_access(req.account_id, Collection::CalendarEvent)?
+                        .assert_has_access(req.from_account_id, Collection::CalendarEvent)?;
+
+                    self.calendar_event_copy(req, access_token, next_call, session)
+                        .await?
+                        .into()
+                }
             },
             RequestMethod::ImportEmail(mut req) => {
                 set_account_id_if_missing(&mut req.account_id, access_token);
@@ -363,12 +573,26 @@ impl RequestHandler for Server {
 
                 self.email_import(req, access_token, session).await?.into()
             }
-            RequestMethod::ParseEmail(mut req) => {
-                set_account_id_if_missing(&mut req.account_id, access_token);
-                access_token.assert_has_access(req.account_id, Collection::Email)?;
+            RequestMethod::Parse(req) => match req {
+                ParseRequestMethod::Email(mut req) => {
+                    set_account_id_if_missing(&mut req.account_id, access_token);
+                    access_token.assert_has_access(req.account_id, Collection::Email)?;
 
-                self.email_parse(req, access_token).await?.into()
-            }
+                    self.email_parse(req, access_token).await?.into()
+                }
+                ParseRequestMethod::ContactCard(mut req) => {
+                    set_account_id_if_missing(&mut req.account_id, access_token);
+                    access_token.assert_has_access(req.account_id, Collection::ContactCard)?;
+
+                    self.contact_card_parse(req, access_token).await?.into()
+                }
+                ParseRequestMethod::CalendarEvent(mut req) => {
+                    set_account_id_if_missing(&mut req.account_id, access_token);
+                    access_token.assert_has_access(req.account_id, Collection::CalendarEvent)?;
+
+                    self.calendar_event_parse(req, access_token).await?.into()
+                }
+            },
             RequestMethod::QueryChanges(req) => self.query_changes(req, access_token).await?.into(),
             RequestMethod::SearchSnippet(mut req) => {
                 set_account_id_if_missing(&mut req.account_id, access_token);
