@@ -17,11 +17,11 @@ use jmap_proto::{
 };
 use jmap_tools::{Key, Value};
 use std::future::Future;
-use store::write::BatchBuilder;
+use store::{ValueKey, write::{AlignedBytes, Archive, BatchBuilder}};
 use trc::AddContext;
 use types::{
     collection::{Collection, SyncCollection},
-    field::Field,
+    field::{Field, IdentityField},
 };
 use utils::sanitize_email;
 
@@ -41,9 +41,8 @@ impl IdentitySet for Server {
     ) -> trc::Result<SetResponse<identity::Identity>> {
         let account_id = request.account_id.document_id();
         let identity_ids = self
-            .get_document_ids(account_id, Collection::Identity)
-            .await?
-            .unwrap_or_default();
+            .document_ids(account_id, Collection::Identity, IdentityField::DocumentId)
+            .await?;
         let mut response = SetResponse::from_request(&request, self.core.jmap.set_max_objects)?;
         let will_destroy = request.unwrap_destroy().into_valid().collect::<Vec<_>>();
 
@@ -111,7 +110,8 @@ impl IdentitySet for Server {
             batch
                 .with_account_id(account_id)
                 .with_collection(Collection::Identity)
-                .create_document(document_id)
+                .with_document(document_id)
+                .tag(IdentityField::DocumentId)
                 .custom(ObjectIndexBuilder::<(), _>::new().with_changes(identity))
                 .caused_by(trc::location!())?
                 .commit_point();
@@ -129,7 +129,12 @@ impl IdentitySet for Server {
             // Obtain identity
             let document_id = id.document_id();
             let identity_ = if let Some(identity_) = self
-                .get_archive(account_id, Collection::Identity, document_id)
+                .store()
+                .get_value::<Archive<AlignedBytes>>(ValueKey::archive(
+                    account_id,
+                    Collection::Identity,
+                    document_id,
+                ))
                 .await?
             {
                 identity_
@@ -157,7 +162,7 @@ impl IdentitySet for Server {
             batch
                 .with_account_id(account_id)
                 .with_collection(Collection::Identity)
-                .update_document(document_id)
+                .with_document(document_id)
                 .custom(
                     ObjectIndexBuilder::new()
                         .with_current(identity)
@@ -176,7 +181,8 @@ impl IdentitySet for Server {
                 batch
                     .with_account_id(account_id)
                     .with_collection(Collection::Identity)
-                    .delete_document(document_id)
+                    .with_document(document_id)
+                    .untag(IdentityField::DocumentId)
                     .clear(Field::ARCHIVE)
                     .log_item_delete(SyncCollection::Identity, None)
                     .commit_point();
