@@ -12,7 +12,7 @@ use common::{
 };
 use email::{
     identity::Identity,
-    message::metadata::MessageMetadata,
+    message::metadata::{ArchivedMetadataHeaderName, ArchivedMetadataHeaderValue, MessageMetadata},
     submission::{Address, Delivered, DeliveryStatus, EmailSubmission, UndoStatus},
 };
 use jmap_proto::{
@@ -28,7 +28,6 @@ use jmap_proto::{
     types::state::State,
 };
 use jmap_tools::{Key, Value};
-use mail_parser::{ArchivedHeaderName, ArchivedHeaderValue};
 use smtp::{
     core::{Session, SessionData},
     queue::spool::SmtpSpool,
@@ -36,7 +35,10 @@ use smtp::{
 use smtp_proto::{MailFrom, RcptTo, request::parser::Rfc5321Parser};
 use std::{borrow::Cow, future::Future};
 use std::{collections::HashMap, sync::Arc, time::Duration};
-use store::write::{BatchBuilder, now};
+use store::{
+    ValueKey,
+    write::{AlignedBytes, Archive, BatchBuilder, now},
+};
 use trc::AddContext;
 use types::{collection::Collection, field::EmailField, id::Id};
 use utils::{map::vec_map::VecMap, sanitize_email};
@@ -95,7 +97,7 @@ impl EmailSubmissionSet for Server {
                     batch
                         .with_account_id(account_id)
                         .with_collection(Collection::EmailSubmission)
-                        .create_document(document_id)
+                        .with_document(document_id)
                         .custom(ObjectIndexBuilder::<(), _>::new().with_changes(submission))
                         .caused_by(trc::location!())?
                         .commit_point();
@@ -118,7 +120,12 @@ impl EmailSubmissionSet for Server {
             // Obtain submission
             let document_id = id.document_id();
             let submission = if let Some(submission) = self
-                .get_archive(account_id, Collection::EmailSubmission, document_id)
+                .store()
+                .get_value::<Archive<AlignedBytes>>(ValueKey::archive(
+                    account_id,
+                    Collection::EmailSubmission,
+                    document_id,
+                ))
                 .await?
             {
                 submission
@@ -171,7 +178,7 @@ impl EmailSubmissionSet for Server {
                         batch
                             .with_account_id(account_id)
                             .with_collection(Collection::EmailSubmission)
-                            .update_document(document_id)
+                            .with_document(document_id)
                             .custom(
                                 ObjectIndexBuilder::new()
                                     .with_current(submission)
@@ -211,14 +218,19 @@ impl EmailSubmissionSet for Server {
         for id in will_destroy {
             let document_id = id.document_id();
             if let Some(submission) = self
-                .get_archive(account_id, Collection::EmailSubmission, document_id)
+                .store()
+                .get_value::<Archive<AlignedBytes>>(ValueKey::archive(
+                    account_id,
+                    Collection::EmailSubmission,
+                    document_id,
+                ))
                 .await?
             {
                 // Update record
                 batch
                     .with_account_id(account_id)
                     .with_collection(Collection::EmailSubmission)
-                    .delete_document(document_id)
+                    .with_document(document_id)
                     .custom(
                         ObjectIndexBuilder::<_, ()>::new().with_current(
                             submission
@@ -456,7 +468,12 @@ impl EmailSubmissionSet for Server {
 
         // Fetch identity's mailFrom
         let identity_mail_from = if let Some(identity) = self
-            .get_archive(account_id, Collection::Identity, submission.identity_id)
+            .store()
+            .get_value::<Archive<AlignedBytes>>(ValueKey::archive(
+                account_id,
+                Collection::Identity,
+                submission.identity_id,
+            ))
             .await?
         {
             identity
@@ -492,12 +509,13 @@ impl EmailSubmissionSet for Server {
 
         // Obtain message metadata
         let metadata_ = if let Some(metadata) = self
-            .get_archive_by_property(
+            .store()
+            .get_value::<Archive<AlignedBytes>>(ValueKey::property(
                 account_id,
                 Collection::Email,
                 submission.email_id,
-                EmailField::Metadata.into(),
-            )
+                EmailField::Metadata,
+            ))
             .await?
         {
             metadata
@@ -516,26 +534,57 @@ impl EmailSubmissionSet for Server {
             for header in metadata.contents[0].parts[0].headers.iter() {
                 if matches!(
                     header.name,
-                    ArchivedHeaderName::To | ArchivedHeaderName::Cc | ArchivedHeaderName::Bcc
+                    ArchivedMetadataHeaderName::To
+                        | ArchivedMetadataHeaderName::Cc
+                        | ArchivedMetadataHeaderName::Bcc
                 ) {
-                    if matches!(header.name, ArchivedHeaderName::Bcc) {
+                    if matches!(header.name, ArchivedMetadataHeaderName::Bcc) {
                         bcc_header = Some(header);
                     }
-                    if let ArchivedHeaderValue::Address(addr) = &header.value {
-                        for address in addr.iter() {
-                            if let Some(address) = address.address().and_then(sanitize_email)
-                                && !rcpt_to.iter().any(|rcpt| rcpt.address == address)
-                            {
-                                submission.envelope.rcpt_to.push(Address {
-                                    email: address.to_string(),
-                                    parameters: None,
-                                });
-                                rcpt_to.push(RcptTo {
-                                    address: Cow::Owned(address),
-                                    ..Default::default()
-                                });
+                    match &header.value {
+                        ArchivedMetadataHeaderValue::AddressList(addr) => {
+                            for address in addr.iter() {
+                                if let Some(address) = address
+                                    .address
+                                    .as_ref()
+                                    .map(|v| v.as_ref())
+                                    .and_then(sanitize_email)
+                                    && !rcpt_to.iter().any(|rcpt| rcpt.address == address)
+                                {
+                                    submission.envelope.rcpt_to.push(Address {
+                                        email: address.to_string(),
+                                        parameters: None,
+                                    });
+                                    rcpt_to.push(RcptTo {
+                                        address: Cow::Owned(address),
+                                        ..Default::default()
+                                    });
+                                }
                             }
                         }
+                        ArchivedMetadataHeaderValue::AddressGroup(groups) => {
+                            for group in groups.iter() {
+                                for address in group.addresses.iter() {
+                                    if let Some(address) = address
+                                        .address
+                                        .as_ref()
+                                        .map(|v| v.as_ref())
+                                        .and_then(sanitize_email)
+                                        && !rcpt_to.iter().any(|rcpt| rcpt.address == address)
+                                    {
+                                        submission.envelope.rcpt_to.push(Address {
+                                            email: address.to_string(),
+                                            parameters: None,
+                                        });
+                                        rcpt_to.push(RcptTo {
+                                            address: Cow::Owned(address),
+                                            ..Default::default()
+                                        });
+                                    }
+                                }
+                            }
+                        }
+                        _ => {}
                     }
                 }
             }
@@ -548,7 +597,7 @@ impl EmailSubmissionSet for Server {
             bcc_header = metadata.contents[0].parts[0]
                 .headers
                 .iter()
-                .find(|header| matches!(header.name, ArchivedHeaderName::Bcc));
+                .find(|header| matches!(header.name, ArchivedMetadataHeaderName::Bcc));
         }
 
         // Update sendAt
@@ -584,8 +633,9 @@ impl EmailSubmissionSet for Server {
         // Remove BCC header if present
         if let Some(bcc_header) = bcc_header {
             let mut new_message = Vec::with_capacity(message.len());
-            new_message.extend_from_slice(&message[..u32::from(bcc_header.offset_field) as usize]);
-            new_message.extend_from_slice(&message[u32::from(bcc_header.offset_end) as usize..]);
+            let range = bcc_header.name_value_range();
+            new_message.extend_from_slice(&message[..range.start]);
+            new_message.extend_from_slice(&message[range.end..]);
             message = new_message;
         }
 
