@@ -4,20 +4,17 @@
  * SPDX-License-Identifier: AGPL-3.0-only OR LicenseRef-SEL
  */
 
+use crate::{
+    Recipient, SpamFilterContext, SpamFilterInput, SpamFilterOutput, SpamFilterResult, TextPart,
+};
+use common::{Server, config::spamfilter::Location};
+use mail_parser::{Header, parsers::MessageStream};
 use std::{
     borrow::Cow,
     hash::{Hash, Hasher},
 };
 
-use common::{Server, config::spamfilter::Location};
-use compact_str::CompactString;
-use mail_parser::{Header, parsers::MessageStream};
-
-use crate::{
-    Recipient, SpamFilterContext, SpamFilterInput, SpamFilterOutput, SpamFilterResult, TextPart,
-};
-
-pub mod bayes;
+pub mod classifier;
 pub mod date;
 pub mod dmarc;
 pub mod domain;
@@ -33,11 +30,9 @@ pub mod pyzor;
 pub mod received;
 pub mod recipient;
 pub mod replyto;
-pub mod reputation;
 pub mod rules;
 pub mod score;
 pub mod subject;
-pub mod trusted_reply;
 pub mod url;
 
 // SPDX-SnippetBegin
@@ -85,7 +80,7 @@ impl SpamFilterContext<'_> {
 }
 
 impl SpamFilterResult {
-    pub fn add_tag(&mut self, tag: impl Into<CompactString>) {
+    pub fn add_tag(&mut self, tag: impl Into<String>) {
         self.tags.insert(tag.into());
     }
 
