@@ -35,6 +35,12 @@ impl From<CompactString> for Value {
     }
 }
 
+impl From<Box<str>> for Value {
+    fn from(value: Box<str>) -> Self {
+        Self::String(CompactString::from(value))
+    }
+}
+
 impl From<u64> for Value {
     fn from(value: u64) -> Self {
         Self::UInt(value)
@@ -53,6 +59,12 @@ impl From<f64> for Value {
     }
 }
 
+impl From<f32> for Value {
+    fn from(value: f32) -> Self {
+        Self::Float(value.into())
+    }
+}
+
 impl From<u16> for Value {
     fn from(value: u16) -> Self {
         Self::UInt(value.into())
