@@ -19,8 +19,10 @@ use jmap_client::{
 };
 use std::{str::FromStr, sync::Arc, time::Duration};
 use store::{
+    ValueKey,
     rand::{self, Rng},
     roaring::RoaringBitmap,
+    write::{AlignedBytes, Archive},
 };
 use types::{collection::Collection, id::Id};
 
@@ -200,28 +202,25 @@ async fn email_tests(server: Server, client: Arc<Client>) {
 
         join_all(futures).await;
 
-        let email_ids = server
-            .get_document_ids(TEST_USER_ID, Collection::Email)
-            .await
-            .unwrap()
-            .unwrap_or_default();
-        let mailbox_ids = server
-            .get_document_ids(TEST_USER_ID, Collection::Mailbox)
-            .await
-            .unwrap()
-            .unwrap_or_default();
+        let cache = server.get_cached_messages(TEST_USER_ID).await.unwrap();
+        let email_ids = cache
+            .emails
+            .items
+            .iter()
+            .map(|e| e.document_id)
+            .collect::<RoaringBitmap>();
+        let mailbox_ids = cache
+            .mailboxes
+            .items
+            .iter()
+            .map(|m| m.document_id)
+            .collect::<RoaringBitmap>();
         assert_eq!(mailbox_ids.len(), 8);
 
         for mailbox in mailboxes.iter() {
             let mailbox_id = Id::from_str(mailbox).unwrap().document_id();
-            let email_ids_in_mailbox = RoaringBitmap::from_iter(
-                server
-                    .get_cached_messages(TEST_USER_ID)
-                    .await
-                    .unwrap()
-                    .in_mailbox(mailbox_id)
-                    .map(|m| m.document_id),
-            );
+            let email_ids_in_mailbox =
+                RoaringBitmap::from_iter(cache.in_mailbox(mailbox_id).map(|m| m.document_id));
             let mut email_ids_check = email_ids_in_mailbox.clone();
             email_ids_check &= &email_ids;
             assert_eq!(email_ids_in_mailbox, email_ids_check);
@@ -230,7 +229,12 @@ async fn email_tests(server: Server, client: Arc<Client>) {
 
             for email_id in &email_ids_in_mailbox {
                 if let Some(mailbox_tags) = server
-                    .get_archive(TEST_USER_ID, Collection::Email, email_id)
+                    .store()
+                    .get_value::<Archive<AlignedBytes>>(ValueKey::archive(
+                        TEST_USER_ID,
+                        Collection::Email,
+                        email_id,
+                    ))
                     .await
                     .unwrap()
                 {
