@@ -4,8 +4,14 @@
  * SPDX-License-Identifier: AGPL-3.0-only OR LicenseRef-SEL
  */
 
-use crate::directory::{DirectoryTest, IntoTestPrincipal, TestPrincipal};
+use std::sync::Arc;
+
+use crate::{
+    directory::{DirectoryTest, IntoTestPrincipal, TestPrincipal},
+    store::cleanup::{store_assert_is_empty, store_destroy},
+};
 use ahash::AHashSet;
+use common::{Core, Inner, Server, config::storage::Storage};
 use directory::{
     Permission, QueryBy, QueryParams, Type,
     backend::{
@@ -17,11 +23,11 @@ use directory::{
         },
     },
 };
+use http::management::stores::destroy_account_data;
 use mail_send::Credentials;
 use store::{
-    BitmapKey, Store, ValueKey,
-    roaring::RoaringBitmap,
-    write::{BatchBuilder, BitmapClass, ValueClass},
+    IterateParams, Store, ValueKey,
+    write::{BatchBuilder, ValueClass},
 };
 use types::collection::Collection;
 
@@ -31,7 +37,7 @@ async fn internal_directory() {
 
     for (store_id, store) in config.stores.stores {
         println!("Testing internal directory with store {:?}", store_id);
-        store.destroy().await;
+        store_destroy(&store).await;
 
         // A principal without name should fail
         assert_eq!(
@@ -655,7 +661,7 @@ async fn internal_directory() {
                     BatchBuilder::new()
                         .with_account_id(account_id)
                         .with_collection(Collection::Email)
-                        .create_document(document_id)
+                        .with_document(document_id)
                         .set(ValueClass::Property(0), "hello".as_bytes())
                         .build_all(),
                 )
@@ -676,7 +682,20 @@ async fn internal_directory() {
         }
 
         // Delete John's account and make sure his records are gone
+        let server = Server {
+            inner: Arc::new(Inner::default()),
+            core: Arc::new(Core {
+                storage: Storage {
+                    data: store.clone(),
+                    blob: store.clone().into(),
+                    fts: store.clone().into(),
+                    ..Default::default()
+                },
+                ..Default::default()
+            }),
+        };
         store.delete_principal(QueryBy::Id(john_id)).await.unwrap();
+        destroy_account_data(&server, john_id, true).await.unwrap();
         assert_eq!(store.get_principal_id("john.doe").await.unwrap(), None);
         assert_eq!(
             store.email_to_id("john.doe@example.org").await.unwrap(),
@@ -707,18 +726,7 @@ async fn internal_directory() {
                 .map(|s| s.into())
                 .collect::<AHashSet<_>>()
         );
-        assert_eq!(
-            store
-                .get_bitmap(BitmapKey {
-                    account_id: john_id,
-                    collection: Collection::Email.into(),
-                    class: BitmapClass::DocumentIds,
-                    document_id: 0
-                })
-                .await
-                .unwrap(),
-            None
-        );
+        assert!(!account_has_emails(&store, john_id).await);
         assert_eq!(
             store
                 .get_value::<String>(ValueKey {
@@ -742,18 +750,7 @@ async fn internal_directory() {
             store.rcpt("jane@example.org").await.unwrap(),
             RcptType::Mailbox
         );
-        assert_eq!(
-            store
-                .get_bitmap(BitmapKey {
-                    account_id: jane_id,
-                    collection: Collection::Email.into(),
-                    class: BitmapClass::DocumentIds,
-                    document_id: 0
-                })
-                .await
-                .unwrap(),
-            Some(RoaringBitmap::from_sorted_iter([document_id]).unwrap())
-        );
+        assert!(account_has_emails(&store, jane_id).await);
         assert_eq!(
             store
                 .get_value::<String>(ValueKey {
@@ -766,6 +763,16 @@ async fn internal_directory() {
                 .unwrap(),
             Some("hello".into())
         );
+
+        // Clean up
+        destroy_account_data(&server, jane_id, true).await.unwrap();
+        for principal_name in ["jane", "list", "sales", "support", "example.org"] {
+            store
+                .delete_principal(QueryBy::Name(principal_name))
+                .await
+                .unwrap();
+        }
+        store_assert_is_empty(&store, store.clone().into(), true).await;
     }
 }
 
@@ -1026,6 +1033,35 @@ impl TestInternalDirectory for Store {
     }
 }
 
+async fn account_has_emails(store: &Store, account_id: u32) -> bool {
+    let mut has_emails = false;
+    store
+        .iterate(
+            IterateParams::new(
+                ValueKey {
+                    account_id,
+                    collection: Collection::Email.into(),
+                    document_id: 0,
+                    class: ValueClass::Property(0),
+                },
+                ValueKey {
+                    account_id,
+                    collection: Collection::Email.into(),
+                    document_id: u32::MAX,
+                    class: ValueClass::Property(u8::MAX),
+                },
+            )
+            .no_values(),
+            |_, _| {
+                has_emails = true;
+                Ok(false)
+            },
+        )
+        .await
+        .unwrap();
+    has_emails
+}
+
 async fn assert_list_members(
     store: &Store,
     list_addr: &str,
