@@ -5,7 +5,7 @@
  */
 
 use crate::{
-    jmap::{ChangeType, IntoJmapSet, JMAPTest, JmapUtils},
+    jmap::{ChangeType, IntoJmapSet, JMAPTest, JmapUtils, wait_for_index},
     webdav::DummyWebDavClient,
 };
 use ahash::AHashSet;
@@ -452,6 +452,7 @@ pub async fn test(params: &mut JMAPTest) {
     }));
 
     // Query tests
+    wait_for_index(&params.server).await;
     assert_eq!(
         account
             .jmap_query(
@@ -631,7 +632,7 @@ END:VCALENDAR
     "timeZone": "Etc/UTC",
     "start": "1996-09-18T14:30:00",
     "status": "confirmed",
-    "iCalComponent": {
+    "iCalendar": {
       "convertedProperties": {
         "duration": {
           "name": "dtend"
