@@ -243,7 +243,7 @@ async fn sieve_scripts() {
     assert_eq!(messages.len(), 2);
     let mut messages = messages.into_iter();
     let notification = messages.next().unwrap();
-    assert_eq!(notification.message.return_path, "");
+    assert_eq!(notification.message.return_path.as_ref(), "");
     assert_eq!(notification.message.recipients.len(), 2);
     assert_eq!(
         notification.message.recipients.first().unwrap().address(),
@@ -323,7 +323,7 @@ async fn sieve_scripts() {
         .await;
 
     let redirect = qr.expect_message().await;
-    assert_eq!(redirect.message.return_path, "");
+    assert_eq!(redirect.message.return_path.as_ref(), "");
     assert_eq!(redirect.message.recipients.len(), 1);
     assert_eq!(
         redirect.message.recipients.first().unwrap().address(),
@@ -351,7 +351,7 @@ async fn sieve_scripts() {
         .await;
 
     let redirect = qr.expect_message().await;
-    assert_eq!(redirect.message.return_path, "");
+    assert_eq!(redirect.message.return_path.as_ref(), "");
     assert_eq!(redirect.message.recipients.len(), 1);
     assert_eq!(
         redirect.message.recipients.first().unwrap().address(),
