@@ -29,7 +29,7 @@ use store::{BlobStore, Store, Stores};
 use tokio::sync::{mpsc, watch};
 use utils::config::Config;
 
-use crate::AssertConfig;
+use crate::{AssertConfig, store::cleanup::store_destroy};
 
 pub mod config;
 pub mod inbound;
@@ -236,7 +236,7 @@ impl TestSMTP {
         let stores = Stores::parse_all(&mut config, false).await;
         let core = Core::parse(&mut config, stores, Default::default()).await;
         let data = Data::parse(&mut config);
-        core.storage.data.destroy().await;
+        store_destroy(&core.storage.data).await;
 
         Self::from_core_and_tempdir(core, data, Some(temp_dir))
     }
